"{ Package: 'stx:goodies/sunit' }"

"{ NameSpace: Smalltalk }"

Object subclass:#TestCaseOutcomeWeakIdentityDictionary
	instanceVariableNames:'contents'
	classVariableNames:''
	poolDictionaries:''
	category:'SUnit-Private'
!

!TestCaseOutcomeWeakIdentityDictionary class methodsFor:'documentation'!

documentation
"
    TestCaseOutcomeWeakIdentityDictionary is a a weak dictionary-like object to store
    key-value pairs (where value is either TestCaseOutcome or another
    Dictionary pointing to TestCaseOutcome). 

    The main difference is that TestCaseOutcomeWeakIdentityDictionary optimizes the case
    it there's only one key-value pair. In this case, key and value is held
    strongly.

    This us used to store remembered outcomes while trying to reduce a number
    of weak objects and thus saving some GC work.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!TestCaseOutcomeWeakIdentityDictionary methodsFor:'accessing'!

at: key
    ^ self at:key ifAbsent:[self errorKeyNotFound:key]

    "Created: / 14-07-2014 / 20:55:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

at: key ifAbsent: block
    contents isNil ifTrue:[ ^ block value ].
    contents class == Association ifTrue:[ 
        contents key == key ifTrue:[ 
            ^ contents value.
        ].
        ^ block value
    ].
    ^ contents at: key ifAbsent: block

    "Created: / 14-07-2014 / 20:55:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

at: key ifAbsentPut: block
    ^ self at: key ifAbsent:[ self at: key put: block value ].

    "Created: / 14-07-2014 / 20:55:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

at: key put: value
    | newContents |

    contents isNil ifTrue:[ 
        contents := Association key: key value: value.
        ^ value.
    ].

    contents class == Association ifTrue:[ 
        contents key == key ifTrue:[ 
            contents value: value.
            ^ value
        ] ifFalse:[
            newContents := self dictionaryClass new.
            newContents at: contents key put: contents value.
            contents := newContents.
        ]
    ].
    contents at: key put: value.
    ^ value

    "Created: / 14-07-2014 / 20:55:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!TestCaseOutcomeWeakIdentityDictionary methodsFor:'private'!

dictionaryClass
    "raise an error: this method should be implemented (TODO)"

    ^ WeakIdentityDictionary

    "Created: / 14-07-2014 / 21:03:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-02-2015 / 00:12:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!TestCaseOutcomeWeakIdentityDictionary class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

